@@ -56,10 +56,10 @@ class reduce_time_cube : public cube {
         if (cube_stref::type_string(in->st_reference()) == "cube_stref_regular") {
             std::shared_ptr<cube_stref_regular> stref = std::dynamic_pointer_cast<cube_stref_regular>(_st_ref);
             duration dt = (stref->t1() - stref->t0() + 1);
-            stref->dt(dt);
+            stref->set_t_axis(stref->t0(), stref->t1(), dt);
         } else if (cube_stref::type_string(_st_ref) == "cube_stref_labeled_time") {
             std::shared_ptr<cube_stref_labeled_time> stref = std::dynamic_pointer_cast<cube_stref_labeled_time>(_st_ref);
-            stref->dt((stref->t1() - stref->t0()) + 1);
+            //stref->dt((stref->t1() - stref->t0()) + 1);
             //stref->t1(stref->t0()) ;  // set nt=1
             stref->set_time_labels({stref->t0()});
         }
