@@ -74,11 +74,15 @@ class window_time_cube : public cube {
 
    public:
     window_time_cube(std::shared_ptr<cube> in, std::vector<std::pair<std::string, std::string>> reducer_bands,
-                     uint16_t win_size_l, uint16_t win_size_r) : cube(std::make_shared<cube_st_reference>(*(in->st_reference()))), _in_cube(in), _reducer_bands(reducer_bands), _win_size_l(win_size_l), _win_size_r(win_size_r), _f(), _band_idx_in(), _kernel() {  // it is important to duplicate st reference here, otherwise changes will affect input cube as well
+                     uint16_t win_size_l, uint16_t win_size_r) : cube(in->st_reference()->copy()), _in_cube(in), _reducer_bands(reducer_bands), _win_size_l(win_size_l), _win_size_r(win_size_r), _f(), _band_idx_in(), _kernel() {  // it is important to duplicate st reference here, otherwise changes will affect input cube as well
         _chunk_size[0] = _in_cube->chunk_size()[0];
         _chunk_size[1] = _in_cube->chunk_size()[1];
         _chunk_size[2] = _in_cube->chunk_size()[2];
 
+        if (!_st_ref->has_regular_time()) {
+            GCBS_WARN("Cube has irregular time dimension, window sizes may vary over time");
+        }
+
         for (uint16_t i = 0; i < reducer_bands.size(); ++i) {
             std::string reducerstr = reducer_bands[i].first;
             std::string bandstr = reducer_bands[i].second;
@@ -93,7 +97,7 @@ class window_time_cube : public cube {
     }
 
     window_time_cube(std::shared_ptr<cube> in, std::vector<double> kernel, uint16_t win_size_l, uint16_t win_size_r)
-        : cube(std::make_shared<cube_st_reference>(*(in->st_reference()))), _in_cube(in), _reducer_bands(), _win_size_l(win_size_l), _win_size_r(win_size_r), _f(), _band_idx_in(), _kernel(kernel) {  // it is important to duplicate st reference here, otherwise changes will affect input cube as well
+        : cube(in->st_reference()->copy()), _in_cube(in), _reducer_bands(), _win_size_l(win_size_l), _win_size_r(win_size_r), _f(), _band_idx_in(), _kernel(kernel) {  // it is important to duplicate st reference here, otherwise changes will affect input cube as well
         _chunk_size[0] = _in_cube->chunk_size()[0];
         _chunk_size[1] = _in_cube->chunk_size()[1];
         _chunk_size[2] = _in_cube->chunk_size()[2];
@@ -119,13 +123,17 @@ class window_time_cube : public cube {
 
     std::shared_ptr<chunk_data> read_chunk(chunkid_t id) override;
 
-    nlohmann::json make_constructible_json() override {
-        nlohmann::json out;
+    json11::Json make_constructible_json() override {
+        json11::Json::object out;
         out["cube_type"] = "window_time";
         if (!_kernel.empty()) {
             out["kernel"] = _kernel;
         } else {
-            out["reducer_bands"] = _reducer_bands;
+            json11::Json::array rb;
+            for (uint16_t i = 0; i < _reducer_bands.size(); ++i) {
+                rb.push_back(json11::Json::array({_reducer_bands[i].first, _reducer_bands[i].second}));
+            }
+            out["reducer_bands"] = rb;
         }
         out["win_size_l"] = _win_size_l;
         out["win_size_r"] = _win_size_r;
@@ -142,17 +150,6 @@ class window_time_cube : public cube {
     std::vector<uint16_t> _band_idx_in;
     std::vector<double> _kernel;
 
-    virtual void set_st_reference(std::shared_ptr<cube_st_reference> stref) override {
-        // copy fields from st_reference type
-        _st_ref->win() = stref->win();
-        _st_ref->srs() = stref->srs();
-        _st_ref->ny() = stref->ny();
-        _st_ref->nx() = stref->nx();
-        _st_ref->t0() = stref->t0();
-        _st_ref->t1() = stref->t1();
-        _st_ref->dt(stref->dt());
-    }
-
     std::function<double(double *buf, uint16_t n)> get_default_reducer_by_name(std::string name);
 
     std::function<double(double *buf, uint16_t n)> get_kernel_reducer(std::vector<double> kernel);
